from si_fab import all as pdk
from si_fab.components.phase_shifter.simulation.simulate import simulate_phaseshifter
import pylab as plt
import numpy as np
import os

name = "exercise_2"
results_array = []

lengths = np.linspace(1e3, 10e3, 6)
vpi_lpi = 1.2
Cl = 1.1e-15  # F/um
R = 50


def phase_unwrap_normalize(optical_out):
    # Unwrap
    phase = np.unwrap(np.angle(optical_out))
    # Normalize (remove phase jump caused by length of waveguide)
    phase_delay = phase[phase != 0.0][0]
    phase[phase == 0.0] = phase_delay
    return (phase - phase_delay) / np.pi


fig = plt.figure()
for ps_l in lengths:
    ps = pdk.PhaseShifterWaveguide(
        length=ps_l,
        core_width=0.45,
        rib_width=8.0,
        junction_offset=-0.1,
    )
    tau = ps.length * Cl * R
    ps.CircuitModel(vpi_lpi=vpi_lpi, tau=tau)

    results = simulate_phaseshifter(
        cell=ps,
        nsteps=500,
        dt=5e-12,
        step_amplitude=1,
        center_wavelength=1.5,
        debug=False,
    )

    results_array.append(results)

    plt.plot(results.timesteps[1:], phase_unwrap_normalize(results["optical_out"][1:]), label=rf"length: {ps_l} um")

plt.legend()
plt.title("Phase shifter step with VpiLpi of 1.2Vcm at 1 Volt swing")
plt.xlabel("Time [s]")
plt.ylabel(r"Phase [$\pi$ rad]")
fig.savefig(os.path.join(f"{name}.png"), bbox_inches="tight")
plt.show()
plt.close(fig)
